package hr.fer.tel.ruazosa.representations;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.LinkedList;
import java.util.List;

import hr.fer.tel.ruazosa.entities.Course;
import hr.fer.tel.ruazosa.entities.Person;
import hr.fer.tel.ruazosa.services.PersonService;

import static hr.fer.tel.ruazosa.representations.RepresentationUtility.badRequestIfNull;

@Service
public class CourseAssembler {
    private PersonAssembler personAssembler;
    private PersonService personService;

    @Autowired
    public CourseAssembler(PersonAssembler personAssembler, PersonService personService) {
        this.personAssembler = personAssembler;
        this.personService = personService;
    }

    public List<ShortCourseResource> toListResource(Iterable<Course> courses) {
        List<ShortCourseResource> resources = new LinkedList<>();
        for(Course c: courses)
            resources.add(toShortResource(c));

        return resources;
    }

    public ShortCourseResource toShortResource(Course course) {
        ShortCourseResource resource = new ShortCourseResource();
        BeanUtils.copyProperties(course, resource);
        return resource;
    }

    public CourseResource toResource(Course course) {
        CourseResource resource = new CourseResource();
        BeanUtils.copyProperties(course, resource);
        Person teacher = course.getTeacher();
        if(teacher != null)
            resource.setTeacher(personAssembler.toShortResource(teacher));
        return resource;
    }

    public Course toEntity(CourseResourceRequest courseResource) {
        Course course = new Course();
        course.setName(courseResource.getName());
        course.setDescription(courseResource.getDescription());
        if(courseResource.getTeacherId() != null) {
            Person teacher = badRequestIfNull(
                    personService.getWithId(courseResource.getTeacherId()),
                    "Teacher does not exist.");
            course.setTeacher(teacher);
        }
        return course;
    }
}
