package hr.fer.tel.ruazosa.services.jpa;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;

import hr.fer.tel.ruazosa.entities.Course;
import hr.fer.tel.ruazosa.entities.Person;
import hr.fer.tel.ruazosa.repositories.CourseRepository;
import hr.fer.tel.ruazosa.repositories.PersonRepository;
import hr.fer.tel.ruazosa.services.CourseService;

@Service
public class CourseServiceJpaImpl implements CourseService {
    private CourseRepository repo;
    private PersonRepository personRepo;

    @Autowired
    public CourseServiceJpaImpl(CourseRepository repo, PersonRepository personRepo) {
        this.repo = repo;
        this.personRepo = personRepo;
    }

    @Override
    public Iterable<Course> findAll() {
        return repo.findAll();
    }

    @Override
    public Course getWithId(Long id) {
        return repo.findOne(id);
    }

    @Override
    @Transactional
    public Course save(Course course) {
        return repo.save(course);
    }

    @Override
    @Transactional
    public void update(Course course) {
        Course oldCourse = repo.findOne(course.getId());
        if(oldCourse == null)
            throw new IllegalArgumentException("Course does not exist.");
        repo.save(course);
    }

    @Override
    @Transactional
    public void delete(Long id) {
        repo.delete(id);
    }

    @Override
    @Transactional
    public void enrollPersonToCourse(Long courseId, Long studentId) {
        Course course = repo.findOne(courseId);
        if(course == null)
            throw new IllegalArgumentException("Course does not exist.");
        Person student = personRepo.findOne(studentId);
        if(student == null)
            throw new IllegalArgumentException("Student can not be null.");
        if(student.equals(course.getTeacher()))
            throw new IllegalArgumentException("Student can not be teacher in the same course.");
        if(course.getStudents().contains(student))
            throw new IllegalArgumentException("Student already enrolled.");

        course.getStudents().add(student);
    }

    @Override
    @Transactional
    public void unenrollPersonToCourse(Long courseId, Long studentId) {
        Course course = repo.findOne(courseId);
        if(course == null)
            throw new IllegalArgumentException("Course does not exist.");
        Person student = personRepo.findOne(studentId);
        if(student == null)
            throw new IllegalArgumentException("Student can not be null.");
        if(!course.getStudents().contains(student))
            throw new IllegalArgumentException("Student not enrolled.");

        course.getStudents().remove(student);
    }
}
