package hr.fer.tel.ruazosa.services.jpa;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;

import hr.fer.tel.ruazosa.entities.Person;
import hr.fer.tel.ruazosa.repositories.PersonRepository;
import hr.fer.tel.ruazosa.services.PersonService;

@Service
public class PersonServiceJpaImpl implements PersonService {

    private PersonRepository repo;

    @Autowired
    public PersonServiceJpaImpl(PersonRepository repo) {
        this.repo = repo;
    }

    @Override
    public Iterable<Person> findAll() {
        return repo.findAll();
    }

    @Override
    public Person getWithId(Long id) {
        return repo.findOne(id);
    }

    @Override
    @Transactional
    public Person save(Person person) {
        return repo.save(person);
    }

    @Override
    @Transactional
    public void update(Person person) {
        Person oldPerson = repo.findOne(person.getId());
        if(oldPerson == null)
            throw new IllegalArgumentException("Person does not exist.");
        repo.save(person);
    }

    @Override
    @Transactional
    public void delete(String id) {
        repo.delete(Long.parseLong(id));
    }
}
